@@ -41,12 +41,9 @@
 #include "fcs_dm.h"
 #include "instance.h"
 
-#ifdef INDIRECT_STACK_STATES
-#include "fcs_hash.h"
-#endif
-
 #include "move_stack_compact_alloc.h"
 #include "inline.h"
+#include "likely.h"
 
 #include "check_limits.h"
 
@@ -76,16 +73,15 @@ static GCC_INLINE ub4 perl_hash_function(
         if (condition_expr)     \
         {      \
             fcs_compact_alloc_release(stacks_allocator);    \
-            new_state_key->stacks[a] = cached_stack;       \
+            new_state->s.stacks[i] = cached_stack;       \
         }
 
 static void GCC_INLINE fc_solve_cache_stacks(
         fc_solve_hard_thread_t * hard_thread,
-        fcs_state_t * new_state_key,
-        fcs_state_extra_info_t * new_state_val
+        fcs_state_keyval_pair_t * new_state
         )
 {
-    int a;
+    int i;
 #if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
     SFO_hash_value_t hash_value_int;
@@ -97,31 +93,31 @@ static void GCC_INLINE fc_solve_cache_stacks(
     char * new_ptr;
     fc_solve_instance_t * instance = hard_thread->instance;
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num = instance->stacks_num;
+    DECLARE_AND_SET_GAME_PARAMS();
 #endif
     fcs_cards_column_t column;
     register int col_len;
     fcs_compact_allocator_t * stacks_allocator;
-    
+
     stacks_allocator = &(hard_thread->allocator);
 
-    for(a=0 ; a < LOCAL_STACKS_NUM ; a++)
+    for(i=0 ; i < LOCAL_STACKS_NUM ; i++)
     {
         /*
          * If the stack is not a copy - it is already cached so skip
          * to the next stack
          * */
-        if (! (new_state_val->stacks_copy_on_write_flags & (1 << a)))
+        if (! (new_state->info.stacks_copy_on_write_flags & (1 << i)))
         {
             continue;
         }
 
-        column = fcs_state_get_col(*new_state_key, a);
+        column = fcs_state_get_col(new_state->s, i);
         col_len = (fcs_col_len(column)+1);
 
         new_ptr = (char*)fcs_compact_alloc_ptr(stacks_allocator, col_len);
         memcpy(new_ptr, column, col_len);
-        new_state_key->stacks[a] = new_ptr;
+        new_state->s.stacks[i] = new_ptr;
 
 #if FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
@@ -129,7 +125,7 @@ static void GCC_INLINE fc_solve_cache_stacks(
         /* This hash function was ripped from the Perl source code.
          * (It is not derived work however). */
         {
-            const char * s_ptr = (char*)(new_state_key->stacks[a]);
+            const char * s_ptr = (char*)(new_state_key->stacks[i]);
             const char * s_end = s_ptr+fcs_col_len(s_ptr)+1;
             hash_value_int = 0;
             while (s_ptr < s_end)
@@ -150,50 +146,56 @@ static void GCC_INLINE fc_solve_cache_stacks(
 #endif
 
         {
-            void * dummy;
-            int verdict;
+            column = fcs_state_get_col(new_state->s, i);
 
-            column = fcs_state_get_col(*new_state_key, a);
-
-            verdict = fc_solve_hash_insert(
+            replace_with_cached(fc_solve_hash_insert(
                 &(instance->stacks_hash),
                 column,
-                column,
                 &cached_stack,
-                &dummy,
                 perl_hash_function(
-                    (ub1 *)new_state_key->stacks[a],
+                    (ub1 *)new_state->s.stacks[i],
                     col_len
                     )
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
                 , hash_value_int
 #endif
-                );
-
-            replace_with_cached(verdict);
+                )
+            );
         }
 
+#elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
+        {
+            column = fcs_state_get_col(new_state->s, i);
+
+            replace_with_cached(
+                fc_solve_columns_google_hash_insert(
+                    instance->stacks_hash,
+                    column,
+                    &cached_stack
+                )
+            );
+        }
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBAVL2_TREE)
 
         cached_stack =
             fcs_libavl2_stacks_tree_insert(
                 instance->stacks_tree,
-                new_state_key->stacks[a]
+                new_state_key->stacks[i]
             );
 
         replace_with_cached(cached_stack != NULL);
 
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBREDBLACK_TREE)
         cached_stack = (void *)rbsearch(
-            new_state_key->stacks[a],
+            new_state_key->stacks[i],
             instance->stacks_tree
             );
 
-        replace_with_cached(cached_stack != new_state_key->stacks[a]);
+        replace_with_cached(cached_stack != new_state_key->stacks[i]);
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_TREE)
         cached_stack = g_tree_lookup(
              instance->stacks_tree,
-             (gpointer)new_state_key->stacks[a]
+             (gpointer)new_state_key->stacks[i]
              );
 
         /* replace_with_cached contains an if statement */
@@ -202,26 +204,26 @@ static void GCC_INLINE fc_solve_cache_stacks(
         {
             g_tree_insert(
                 instance->stacks_tree,
-                (gpointer)new_state_key->stacks[a],
-                (gpointer)new_state_key->stacks[a]
+                (gpointer)new_state_key->stacks[i],
+                (gpointer)new_state_key->stacks[i]
                 );
         }
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_HASH)
         cached_stack = g_hash_table_lookup(
             instance->stacks_hash,
-            (gconstpointer)new_state_key->stacks[a]
+            (gconstpointer)new_state_key->stacks[i]
             );
         replace_with_cached(cached_stack != NULL)
         else
         {
             g_hash_table_insert(
                 instance->stacks_hash,
-                (gpointer)new_state_key->stacks[a],
-                (gpointer)new_state_key->stacks[a]
+                (gpointer)new_state_key->stacks[i],
+                (gpointer)new_state_key->stacks[i]
                 );
         }
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)
-        column = fcs_state_get_col(*new_state_key, a);
+        column = fcs_state_get_col(*new_state_key, i);
 
         JHSI(
             PValue,
@@ -248,7 +250,7 @@ static void GCC_INLINE fc_solve_cache_stacks(
 
 #else /* #ifdef INDIRECT_STACK_STATES */
 
-#define fc_solve_cache_stacks(hard_thread, new_state_key, new_state_val) \
+#define fc_solve_cache_stacks(hard_thread, new_state_key) \
     {}
 
 #endif
@@ -321,18 +323,54 @@ guint fc_solve_hash_function(gconstpointer key)
  *    that were already checked.
  *    If it is:
  *
- *        5a. Return FCS_STATE_ALREADY_EXISTS
+ *        5a. Return FALSE.
  *
  *    If it isn't:
  *
- *        5b. Add the new state and return FCS_STATE_DOES_NOT_EXIST
+ *        5b. Add the new state and return TRUE.
  * */
 
-GCC_INLINE int fc_solve_check_and_add_state(
-    fc_solve_soft_thread_t * soft_thread,
-    fcs_state_extra_info_t * new_state_val,
-    fcs_state_extra_info_t * * existing_state_val
+static GCC_INLINE void on_state_new(
+    fc_solve_instance_t * instance,
+    fc_solve_hard_thread_t * hard_thread,
+    fcs_collectible_state_t * new_state
+)
+{
+    register fcs_collectible_state_t * parent_state;
+
+    /* The new state was not found in the cache, and it was already inserted */
+    if (likely(parent_state = FCS_S_PARENT(new_state)))
+    {
+        (FCS_S_NUM_ACTIVE_CHILDREN(parent_state))++;
+        /* If parent_val is defined, so is moves_to_parent */
+        FCS_S_MOVES_TO_PARENT(new_state) =
+            fc_solve_move_stack_compact_allocate(
+                hard_thread,
+                FCS_S_MOVES_TO_PARENT(new_state)
+            );
+    }
+
+    instance->active_num_states_in_collection++;
+    instance->num_states_in_collection++;
+
+    return;
+}
+
+#ifdef FCS_RCS_STATES
+fcs_bool_t fc_solve_check_and_add_state(
+    fc_solve_hard_thread_t * hard_thread,
+    fcs_state_t * new_state_key,
+    fcs_collectible_state_t * new_state,
+    fcs_state_t * * existing_state_key,
+    fcs_collectible_state_t * * existing_state
     )
+#else
+fcs_bool_t fc_solve_check_and_add_state(
+    fc_solve_hard_thread_t * hard_thread,
+    fcs_collectible_state_t * new_state,
+    fcs_collectible_state_t * * existing_state
+    )
+#endif
 {
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
@@ -341,29 +379,26 @@ GCC_INLINE int fc_solve_check_and_add_state(
 #endif
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
     fcs_standalone_state_ptrs_t * pos_ptr;
-    int found;
+    fcs_bool_t found;
 #endif
-    fc_solve_hard_thread_t * hard_thread = soft_thread->hard_thread;
     fc_solve_instance_t * instance = hard_thread->instance;
-    fcs_state_t * new_state_key = new_state_val->key;
-
-    int is_state_new;
-
-    if (check_if_limits_exceeded())
-    {
-        return FCS_STATE_BEGIN_SUSPEND_PROCESS;
-    }
 
+    /* #if'ing out because it doesn't belong here. */
+#if 0
     if ((instance->max_depth >= 0) &&
         (new_state_val->depth >= instance->max_depth))
     {
         return FCS_STATE_EXCEEDS_MAX_DEPTH;
     }
+#endif
 
+    fc_solve_cache_stacks(hard_thread, new_state);
 
-    fc_solve_cache_stacks(hard_thread, new_state_key, new_state_val);
-
-    fc_solve_canonize_state(new_state_val,
+    fc_solve_canonize_state(
+#ifdef FCS_RCS_STATES
+            new_state_key,
+#endif
+            new_state,
             INSTANCE_FREECELLS_NUM,
             INSTANCE_STACKS_NUM
             );
@@ -398,24 +433,56 @@ GCC_INLINE int fc_solve_check_and_add_state(
     }
 #endif
     {
-        void * existing_key_void, * existing_val_void;
-    is_state_new = (fc_solve_hash_insert(
+        void * existing_void;
+        if (fc_solve_hash_insert(
         &(instance->hash),
+        new_state,
+#ifdef FCS_RCS_STATES
         new_state_key,
-        new_state_val,
-        &existing_key_void,
-        &existing_val_void,
+#endif
+        &existing_void,
         perl_hash_function(
-            (ub1 *)new_state_key,
+#ifdef FCS_RCS_STATES
+            (ub1 *)(new_state_key),
             sizeof(*new_state_key)
+#else
+            (ub1 *)&(new_state->s),
+            sizeof(new_state->s)
+#endif
             )
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
         , hash_value_int
 #endif
-        ) == 0);
-        if (! is_state_new)
+        ))
         {
-            *existing_state_val = existing_val_void;
+            *existing_state = existing_void;
+            return FALSE;
+        }
+        else
+        {
+            on_state_new(instance, hard_thread, new_state);
+            return TRUE;
+        }
+    }
+#elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GOOGLE_DENSE_HASH)
+    {
+        void * existing_void;
+
+        /*  TODO : check if this condition should be negated. */
+        if (fc_solve_states_google_hash_insert(
+            instance->hash,
+            new_state,
+            &(existing_void)
+            )
+        )
+        {
+            *existing_state = existing_void;
+            return FALSE;
+        }
+        else
+        {
+            on_state_new(instance, hard_thread, new_state);
+            return TRUE;
         }
     }
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
@@ -438,7 +505,7 @@ GCC_INLINE int fc_solve_check_and_add_state(
 
         if (found)
         {
-            is_state_new = 0;
+            is_state_new = FALSE;
             *existing_state_val = pos_ptr->val;
         }
         else
@@ -460,7 +527,7 @@ GCC_INLINE int fc_solve_check_and_add_state(
             {
                 /* The sort margin is full, let's combine it with the main array */
 
-                instance->indirect_prev_states = 
+                instance->indirect_prev_states =
                     realloc(
                         instance->indirect_prev_states,
                         sizeof(instance->indirect_prev_states[0])
@@ -483,14 +550,14 @@ GCC_INLINE int fc_solve_check_and_add_state(
 
                 instance->num_prev_states_margin=0;
             }
-            is_state_new = 1;
+            is_state_new = TRUE;
         }
 
     }
     else
     {
         *existing_state_val = pos_ptr->val;
-        is_state_new = 0;
+        is_state_new = FALSE;
     }
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
@@ -500,14 +567,24 @@ GCC_INLINE int fc_solve_check_and_add_state(
     is_state_new = ((*existing_state_val) == new_state_val);
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL2_TREE)
+    instance->tree_new_state_key = new_state_key;
+    instance->tree_new_state = new_state;
 
-    *existing_state_val = (fcs_state_extra_info_t *)
-        fcs_libavl2_states_tree_insert(instance->tree, new_state_val);
-    is_state_new = ((*existing_state_val) == NULL);
-
+    if ((*existing_state = (fcs_collectible_state_t *)
+        fcs_libavl2_states_tree_insert(instance->tree, new_state))
+            == NULL
+       )
+    {
+        on_state_new(instance, hard_thread, new_state);
+        return TRUE;
+    }
+    else
+    {
+        return FALSE;
+    }
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_TREE)
     *existing_state_val = g_tree_lookup(instance->tree, (gpointer)new_state_key);
-    if (*existing_state_val == NULL)
+    if ((is_state_new = (*existing_state_val == NULL)))
     {
         /* The new state was not found. Let's insert it.
          * The value must be the same as the key, so g_tree_lookup()
@@ -517,19 +594,12 @@ GCC_INLINE int fc_solve_check_and_add_state(
             (gpointer)new_state_key,
             (gpointer)new_state_val
             );
-        is_state_new = 1;
     }
-    else
-    {
-        is_state_new = 0;
-    }
-
-
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
-    *existing_state_val = g_hash_table_lookup(instance->hash, 
+    *existing_state_val = g_hash_table_lookup(instance->hash,
             (gpointer)new_state_key);
-    if (*existing_state_val == NULL)
+    if ((is_state_new = (*existing_state_val == NULL)))
     {
         /* The new state was not found. Let's insert it.
          * The value must be the same as the key, so g_tree_lookup()
@@ -540,11 +610,6 @@ GCC_INLINE int fc_solve_check_and_add_state(
             (gpointer)new_state_val
 
             );
-        is_state_new = 1;
-    }
-    else
-    {
-        is_state_new = 0;
     }
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_DB_FILE)
@@ -552,13 +617,13 @@ GCC_INLINE int fc_solve_check_and_add_state(
         DBT key, value;
         key.data = new_state;
         key.size = sizeof(*new_state);
-        if (instance->db->get(
+        if ((is_state_new = (instance->db->get(
             instance->db,
             NULL,
             &key,
             &value,
             0
-            ) == 0)
+            ) == 0)))
         {
             /* The new state was not found. Let's insert it.
              * The value must be the same as the key, so g_tree_lookup()
@@ -572,11 +637,6 @@ GCC_INLINE int fc_solve_check_and_add_state(
                 &key,
                 &value,
                 0);
-            is_state_new = 1;
-        }
-        else
-        {
-            is_state_new = 0;
         }
     }
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_JUDY)
@@ -586,46 +646,20 @@ GCC_INLINE int fc_solve_check_and_add_state(
         JHSI(PValue, instance->judy_array, new_state_key, sizeof(*new_state_key));
 
         /* later_todo : Handle out-of-memory. */
-        if (*PValue == 0)
+        if ((is_state_new = (*PValue == 0)))
         {
             /* A new state. */
-            is_state_new = 1;
             *PValue = (PWord_t)(*existing_state_val = new_state_val);
         }
         else
         {
             /* Already exists. */
-            is_state_new = 0;
             *existing_state_val = (fcs_state_extra_info_t *)(*PValue);
         }
     }
 #else
 #error no define
 #endif
-    if (is_state_new)
-    {
-        /* The new state was not found in the cache, and it was already inserted */
-        if (new_state_val->parent_val)
-        {
-            new_state_val->parent_val->num_active_children++;
-        }
-        instance->num_states_in_collection++;
-
-        if (new_state_val->moves_to_parent != NULL)
-        {
-            new_state_val->moves_to_parent =
-                fc_solve_move_stack_compact_allocate(
-                    hard_thread,
-                    new_state_val->moves_to_parent
-                    );
-        }
-
-        return FCS_STATE_DOES_NOT_EXIST;
-    }
-    else
-    {
-        return FCS_STATE_ALREADY_EXISTS;
-    }
 }
 
 #endif /* #ifndef FC_SOLVE__CAAS_C */
