@@ -22,7 +22,7 @@
  * OTHER DEALINGS IN THE SOFTWARE.
  */
 /*
- * cmd_line.c - the Freecell Solver command line arguments-like parsing 
+ * cmd_line.c - the Freecell Solver command line arguments-like parsing
  * routines. Useful for more easily configuring a Freecell Solver instance.
  */
 
@@ -42,24 +42,24 @@
 
 #include "cmd_line_enum.h"
 
-static GCC_INLINE int read_preset(char * preset_name, args_man_t * * args_man, char * * opened_files_dir_to_assign, char * user_preset_dir)
+static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_man, char * * opened_files_dir_to_assign, const char * user_preset_dir)
 {
     int ret_code = 1;
     char * home_dir_presetrc = NULL, * global_presetrc = NULL, * env_var_presetrc = NULL;
-    char * path;
-    char * * presetrc_pathes[5];
+    const char * path;
+    const char * * presetrc_pathes[5];
     int path_idx;
     char line[8192];
     FILE * f = NULL;
     char * opened_files_dir = NULL;
     int read_next_preset = 0;
-   
+
 
     {
         register int idx = 0;
-        presetrc_pathes[idx++] = &env_var_presetrc;
-        presetrc_pathes[idx++] = &home_dir_presetrc;
-        presetrc_pathes[idx++] = &global_presetrc;
+        presetrc_pathes[idx++] = (const char * *)&env_var_presetrc;
+        presetrc_pathes[idx++] = (const char * *)&home_dir_presetrc;
+        presetrc_pathes[idx++] = (const char * *)&global_presetrc;
         presetrc_pathes[idx++] = &user_preset_dir;
         presetrc_pathes[idx++] = NULL;
     }
@@ -171,19 +171,20 @@ have_preset:
 DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     void * instance,
     int argc,
-    char * argv[],
+    freecell_solver_str_t argv[],
     int start_arg,
-    char * * known_parameters,
+    freecell_solver_str_t * known_parameters,
     freecell_solver_user_cmd_line_known_commands_callback_t callback,
     void * callback_context,
     char * * error_string,
     int * last_arg,
     int file_nesting_count,
-    char * opened_files_dir
+    freecell_solver_str_t opened_files_dir
     )
 {
-    char * * arg, * * arg_argc;
-    char * * known_param;
+    freecell_solver_str_t * arg;
+    freecell_solver_str_t * arg_argc;
+    freecell_solver_str_t * known_param;
     int num_to_skip;
     int callback_ret;
     int opt;
@@ -232,7 +233,7 @@ case 'a':
 {
 if (!strncmp(p, "-star-weights", 13)) {
 p += 13;
-opt = FCS_OPT_A_STAR_WEIGHTS;
+opt = FCS_OPT_BEFS_WEIGHTS;
 
 }
 }
@@ -240,9 +241,26 @@ opt = FCS_OPT_A_STAR_WEIGHTS;
 break;
 
 case 'c':
+
 {
-if (!strncmp(p, "alc-real-depth", 14)) {
-p += 14;
+if (*(p++) == 'a')
+{
+{ switch(*(p++)) {
+case 'c':
+{
+if (!strncmp(p, "he-limit", 8)) {
+p += 8;
+opt = FCS_OPT_CACHE_LIMIT;
+
+}
+}
+
+break;
+
+case 'l':
+{
+if (!strncmp(p, "c-real-depth", 12)) {
+p += 12;
 opt = FCS_OPT_CALC_REAL_DEPTH;
 
 }
@@ -250,10 +268,25 @@ opt = FCS_OPT_CALC_REAL_DEPTH;
 
 break;
 
+}
+}
+
+}
+
+}
+
+break;
+
 case 'd':
+
 {
-if (!strncmp(p, "ecks-num", 8)) {
-p += 8;
+if (*(p++) == 'e')
+{
+{ switch(*(p++)) {
+case 'c':
+{
+if (!strncmp(p, "ks-num", 6)) {
+p += 6;
 opt = FCS_OPT_DECKS_NUM;
 
 }
@@ -261,6 +294,26 @@ opt = FCS_OPT_DECKS_NUM;
 
 break;
 
+case 'p':
+{
+if (!strncmp(p, "th-tests-order", 14)) {
+p += 14;
+opt = FCS_OPT_DEPTH_TESTS_ORDER;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+
+}
+
+break;
+
 case 'e':
 {
 if (!strncmp(p, "mpty-stacks-filled-by", 21)) {
@@ -273,9 +326,46 @@ opt = FCS_OPT_EMPTY_STACKS_FILLED_BY;
 break;
 
 case 'f':
+{ switch(*(p++)) {
+case 'l':
 {
-if (!strncmp(p, "reecells-num", 12)) {
-p += 12;
+if (!strncmp(p, "are", 3)) {
+p += 3;
+{ switch(*(p++)) {
+case '-':
+{
+if (!strncmp(p, "name", 4)) {
+p += 4;
+opt = FCS_OPT_FLARE_NAME;
+
+}
+}
+
+break;
+
+case 's':
+{
+if (!strncmp(p, "-plan", 5)) {
+p += 5;
+opt = FCS_OPT_FLARES_PLAN;
+
+}
+}
+
+break;
+
+}
+}
+
+}
+}
+
+break;
+
+case 'r':
+{
+if (!strncmp(p, "eecells-num", 11)) {
+p += 11;
 opt = FCS_OPT_FREECELLS_NUM;
 
 }
@@ -283,6 +373,11 @@ opt = FCS_OPT_FREECELLS_NUM;
 
 break;
 
+}
+}
+
+break;
+
 case 'g':
 {
 if (!strncmp(p, "ame", 3)) {
@@ -374,6 +469,17 @@ case 'n':
 if (!strncmp(p, "ext-", 4)) {
 p += 4;
 { switch(*(p++)) {
+case 'f':
+{
+if (!strncmp(p, "lare", 4)) {
+p += 4;
+opt = FCS_OPT_NEXT_FLARE;
+
+}
+}
+
+break;
+
 case 'h':
 {
 if (!strncmp(p, "ard-thread", 10)) {
@@ -547,9 +653,11 @@ opt = FCS_OPT_SCANS_SYNERGY;
 break;
 
 case 'e':
+{ switch(*(p++)) {
+case 'q':
 {
-if (!strncmp(p, "quence", 6)) {
-p += 6;
+if (!strncmp(p, "uence", 5)) {
+p += 5;
 { switch(*(p++)) {
 case '-':
 {
@@ -581,6 +689,22 @@ break;
 
 break;
 
+case 't':
+{
+if (!strncmp(p, "-pruning", 8)) {
+p += 8;
+opt = FCS_OPT_SET_PRUNING;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
 case 'o':
 {
 if (!strncmp(p, "ft-thread-step", 14)) {
@@ -627,9 +751,11 @@ break;
 break;
 
 case 't':
+{ switch(*(p++)) {
+case 'e':
 {
-if (!strncmp(p, "ests-order", 10)) {
-p += 10;
+if (!strncmp(p, "sts-order", 9)) {
+p += 9;
 opt = FCS_OPT_TESTS_ORDER;
 
 }
@@ -637,6 +763,22 @@ opt = FCS_OPT_TESTS_ORDER;
 
 break;
 
+case 'r':
+{
+if (!strncmp(p, "im-max-stored-states", 20)) {
+p += 20;
+opt = FCS_OPT_TRIM_MAX_STORED_STATES;
+
+}
+}
+
+break;
+
+}
+}
+
+break;
+
 }
 }
 
@@ -646,7 +788,18 @@ case 'a':
 {
 if (!strncmp(p, "sw", 2)) {
 p += 2;
-opt = FCS_OPT_A_STAR_WEIGHTS;
+opt = FCS_OPT_BEFS_WEIGHTS;
+
+}
+}
+
+break;
+
+case 'd':
+{
+if (!strncmp(p, "to", 2)) {
+p += 2;
+opt = FCS_OPT_DEPTH_TESTS_ORDER;
 
 }
 }
@@ -733,6 +886,18 @@ break;
 
 case 'n':
 { switch(*(p++)) {
+case 'f':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_NEXT_FLARE;
+}
+}
+
+break;
+
 case 'h':
 {
 if (!strncmp(p, "t", 1)) {
@@ -816,6 +981,18 @@ opt = FCS_OPT_SEED;
 
 break;
 
+case 'p':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_SET_PRUNING;
+}
+}
+
+break;
+
 case 't':
 {
 if (!strncmp(p, "ep", 2)) {
@@ -833,10 +1010,29 @@ break;
 break;
 
 case 't':
+{ switch(*(p++)) {
+case 'm':
 {
-if (!strncmp(p, "o", 1)) {
-p += 1;
+if (!strncmp(p, "ss", 2)) {
+p += 2;
+opt = FCS_OPT_TRIM_MAX_STORED_STATES;
+
+}
+}
+
+break;
+
+case 'o':
+
+{
+if (*p == '\0')
+{
+
 opt = FCS_OPT_TESTS_ORDER;
+}
+}
+
+break;
 
 }
 }
@@ -1150,17 +1346,34 @@ break;
         }
         break;
 
-        case FCS_OPT_A_STAR_WEIGHTS: /* STRINGS=-asw|--a-star-weights; */
+        case FCS_OPT_BEFS_WEIGHTS: /* STRINGS=-asw|--a-star-weights; */
         {
             PROCESS_OPT_ARG() ;
 
             {
                 int a;
-                char * start_num;
-                char * end_num;
-                char save;
+                freecell_solver_str_t start_num;
+                freecell_solver_str_t end_num;
+                char * substring_copy;
+
                 start_num = (*arg);
-                for(a=0;a<5;a++)
+
+                substring_copy = malloc(strlen(start_num) + 1);
+
+                /* Initialize all the Best Frist Search weights at first
+                 * to 0 so
+                 * we won't have partial initialization.
+                 * */
+                for (a=0 ; a<5 ; a++)
+                {
+                    freecell_solver_user_set_a_star_weight(
+                        instance,
+                        a,
+                        0
+                        );
+                }
+
+                for (a=0 ; a<5 ; a++)
                 {
                     while ((*start_num > '9') && (*start_num < '0') && (*start_num != '\0'))
                     {
@@ -1175,16 +1388,25 @@ break;
                     {
                         end_num++;
                     }
-                    save = *end_num;
-                    *end_num = '\0';
+
+                    strncpy(substring_copy, start_num, end_num-start_num);
+                    substring_copy[end_num-start_num] = '\0';
                     freecell_solver_user_set_a_star_weight(
                         instance,
                         a,
-                        atof(start_num)
+                        atof(substring_copy)
                         );
-                    *end_num = save;
+                    /* Make sure that if the string terminated here -
+                     * we stop.
+                     * */
+                    if (*end_num == '\0')
+                    {
+                        break;
+                    }
                     start_num=end_num+1;
                 }
+
+                free(substring_copy);
             }
         }
         break;
@@ -1214,10 +1436,28 @@ break;
         }
         break;
 
+        case FCS_OPT_TRIM_MAX_STORED_STATES: /* STRINGS=-tmss|--trim-max-stored-states; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_set_stored_states_trimming_limit(
+                instance,
+                atol((*arg))
+            );
+        }
+        break;
+
         case FCS_OPT_NEXT_INSTANCE: /* STRINGS=-ni|--next-instance; */
         {
             freecell_solver_user_next_instance(instance);
         }
+        break;
+
+        case FCS_OPT_NEXT_FLARE: /* STRINGS=-nf|--next-flare; */
+        {
+            freecell_solver_user_next_flare(instance);
+        }
+        break;
 
         case FCS_OPT_NEXT_SOFT_THREAD: /* STRINGS=-nst|--next-soft-thread; */
         case FCS_OPT_NEXT_HARD_THREAD: /* STRINGS=-nht|--next-hard-thread; */
@@ -1279,6 +1519,14 @@ break;
         }
         break;
 
+        case FCS_OPT_FLARE_NAME: /* STRINGS=--flare-name; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_flare_name(instance, (*arg));
+        }
+        break;
+
         case FCS_OPT_PRELUDE: /* STRINGS=--prelude; */
         {
             PROCESS_OPT_ARG() ;
@@ -1287,6 +1535,14 @@ break;
         }
         break;
 
+        case FCS_OPT_FLARES_PLAN: /* STRINGS=--flares-plan; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_flares_plan(instance, (*arg));
+        }
+        break;
+
         case FCS_OPT_OPTIMIZATION_TESTS_ORDER: /* STRINGS=-opt-to|--optimization-tests-order; */
         {
             int ret;
@@ -1366,7 +1622,8 @@ break;
             }
             else
             {
-                char * s, * buffer;
+                freecell_solver_str_t s;
+                char * buffer;
                 FILE * f;
                 long file_len;
                 int ret;
@@ -1465,7 +1722,7 @@ break;
                     ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                         instance,
                         args_man->argc - num_file_args_to_skip,
-                        args_man->argv + num_file_args_to_skip,
+                        (freecell_solver_str_t *)(args_man->argv + num_file_args_to_skip),
                         0,
                         known_parameters,
                         callback,
@@ -1517,7 +1774,7 @@ break;
                     ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                         instance,
                         preset_args->argc,
-                        preset_args->argv,
+                        (freecell_solver_str_t *)preset_args->argv,
                         0,
                         known_parameters,
                         callback,
@@ -1547,6 +1804,95 @@ break;
         }
         break;
 
+        case FCS_OPT_DEPTH_TESTS_ORDER: /* STRINGS=-dto|--depth-tests-order; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            {
+                char * fcs_user_errstr;
+                int min_depth;
+                int ret;
+                const char * s;
+
+                min_depth = 0;
+
+                s = (*arg);
+                while(isdigit(*s))
+                {
+                    s++;
+                }
+                if (*s == ',')
+                {
+                    min_depth = atoi((*arg));
+                    s++;
+                }
+
+                ret = freecell_solver_user_set_depth_tests_order(
+                        instance,
+                        min_depth,
+                        (*arg),
+                        &fcs_user_errstr
+                        );
+
+                if (ret != 0)
+                {
+                    char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                    sprintf(
+                            errstr,
+                            "Error in depth tests' order!\n%s\n",
+                            fcs_user_errstr
+                           );
+                    free(fcs_user_errstr);
+
+                    *error_string = errstr;
+
+                    RET_ERROR_IN_ARG() ;
+                }
+            }
+        }
+        break;
+
+        case FCS_OPT_SET_PRUNING: /* STRINGS=-sp|--set-pruning; */
+        {
+            int ret;
+            char * fcs_user_errstr;
+
+            PROCESS_OPT_ARG() ;
+
+            ret = freecell_solver_user_set_pruning(
+                    instance,
+                    (*arg),
+                    &fcs_user_errstr
+                    );
+
+            if (ret != 0)
+            {
+                char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                sprintf(
+                    errstr,
+                    "Error in the optimization scan's pruning!\n%s\n",
+                    fcs_user_errstr
+                    );
+                free(fcs_user_errstr);
+
+                *error_string = errstr;
+
+                RET_ERROR_IN_ARG() ;
+            }
+        }
+        break;
+
+        case FCS_OPT_CACHE_LIMIT: /* STRINGS=--cache-limit; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_cache_limit(
+                instance,
+                atol(*arg)
+            );
+        }
+        break;
+
 
         }
         /* OPT-PARSE-END */
@@ -1559,9 +1905,9 @@ break;
 DLLEXPORT int freecell_solver_user_cmd_line_parse_args(
     void * instance,
     int argc,
-    char * argv[],
+    freecell_solver_str_t argv[],
     int start_arg,
-    char * * known_parameters,
+    freecell_solver_str_t * known_parameters,
     freecell_solver_user_cmd_line_known_commands_callback_t callback,
     void * callback_context,
     char * * error_string,
