@@ -25,7 +25,7 @@
  * main.c - the main() and related functions of the fc-solve command line
  * executable.
  *
- * It is documented in the documents "README", "USAGE", etc. in the 
+ * It is documented in the documents "README", "USAGE", etc. in the
  * Freecell Solver distribution from http://fc-solve.berlios.de/ .
  */
 #include <string.h>
@@ -35,21 +35,31 @@
 
 #include "fcs_cl.h"
 #include "unused.h"
+#include "bool.h"
+
+enum STANDARD_NOTATION_TYPE
+{
+    STANDARD_NOTATION_NO,
+    STANDARD_NOTATION_REGULAR,
+    STANDARD_NOTATION_EXTENDED
+};
+
 
 struct fc_solve_display_information_context_struct
 {
-    int debug_iter_state_output;
+    fcs_bool_t debug_iter_state_output;
     int freecells_num;
     int stacks_num;
     int decks_num;
-    int parseable_output;
-    int canonized_order_output;
-    int display_10_as_t;
-    int display_parent_iter_num;
-    int debug_iter_output_on;
-    int display_moves;
-    int display_states;
+    fcs_bool_t parseable_output;
+    fcs_bool_t canonized_order_output;
+    fcs_bool_t display_10_as_t;
+    fcs_bool_t display_parent_iter_num;
+    fcs_bool_t debug_iter_output_on;
+    fcs_bool_t display_moves;
+    fcs_bool_t display_states;
     int standard_notation;
+    const char * output_filename;
 };
 
 typedef struct fc_solve_display_information_context_struct fc_solve_display_information_context_t;
@@ -58,18 +68,17 @@ static void init_debug_context(
     fc_solve_display_information_context_t * dc
     )
 {
-    dc->debug_iter_state_output = 0;
-    dc->parseable_output = 0;
-    dc->canonized_order_output = 0;
-    dc->display_10_as_t = 0;
-    dc->display_parent_iter_num = 0;
-    dc->display_moves = 0;
-    dc->display_states = 1;
-    dc->standard_notation = 0;
+    dc->debug_iter_state_output = FALSE;
+    dc->parseable_output = FALSE;
+    dc->canonized_order_output = FALSE;
+    dc->display_10_as_t = FALSE;
+    dc->display_parent_iter_num = FALSE;
+    dc->display_moves = FALSE;
+    dc->display_states = TRUE;
+    dc->standard_notation = STANDARD_NOTATION_NO;
+    dc->output_filename = NULL;
 }
 
-
-
 static void my_iter_handler(
     void * user_instance,
     int iter_num,
@@ -87,6 +96,9 @@ static void my_iter_handler(
     fprintf(stdout, "Stored-States: %i\n",
         freecell_solver_user_get_num_states_in_collection(user_instance)
         );
+    fprintf(stdout, "Scan: %s\n",
+        freecell_solver_user_get_current_soft_thread_name(user_instance)
+        );
     if (context->display_parent_iter_num)
     {
         fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
@@ -120,12 +132,12 @@ help_screen_t help_screens[] = {
     "configs",
 "These configurations are usually faster than the unmodified run:\n"
 "\n"
-"    fc-solve -l cool-jives\n"
-"    fc-solve -l john-galt-line\n"
+"    fc-solve -l foss-nessy\n"
+"    fc-solve -l the-iglu-cabal\n"
 "\n"
 "Or if you want an accurate verdict:\n"
 "\n"
-"    fc-solve -l fools-gold\n"
+"    fc-solve -l toons-for-twenty-somethings\n"
 "\n"
 "If you want to try constructing your own configurations refer to the\n"
 "USAGE file in the Freecell Solver distribution\n"
@@ -143,6 +155,8 @@ help_screen_t help_screens[] = {
 "     display the summary help screen\n"
 "--version\n"
 "     display the version number of the components\n"
+"-o [Filename] --output [Filename]\n"
+"     Redirect output to a file.\n"
 "-i     --iter-output\n"
 "     display the iteration number and depth in every state that is checked\n"
 "-s     --state-output\n"
@@ -255,19 +269,23 @@ help_screen_t help_screens[] = {
 "        Tests are grouped with parenthesis or square brackets. Each group\n"
 "        will be randomized as a whole by the random-dfs scan.\n"
 "\n"
+"-dto [min_d],[tests_order]   --depth-tests-order [min_d],[tests_order]\n"
+"     Like --to / --tests-order only assigns the tests starting from the\n"
+"     minimal depth.\n"
+"\n"
 "\n"
 "-me [solving_method]   --method [solving_method]\n"
 "     Specify a solving method. Available methods are:\n"
-"        \"a-star\" - A*\n"
+"        \"a-star\" - Best-First-Search\n"
 "        \"bfs\" - Breadth-First Search\n"
 "        \"dfs\" - Depth-First Search (default)\n"
 "        \"random-dfs\" - A randomized DFS\n"
 "        \"soft-dfs\" - \"Soft\" DFS\n"
 "\n"
-"-asw [A* Weights]   --a-star-weight [A* Weights]\n"
-"     Specify weights for the A* scan, assuming it is used. The parameter\n"
-"     should be a comma-separated list of numbers, each one is proportional\n"
-"     to the weight of its corresponding test.\n"
+"-asw [BeFS Weights]   --a-star-weight [BeFS Weights]\n"
+"     Specify weights for the Best-first-search scan, assuming it is used.\n"
+"     The parameter should be a comma-separated list of numbers, each one is\n"
+"     proportional to the weight of its corresponding test.\n"
 "\n"
 "     The numbers are, in order:\n"
 "     1. The number of cards out.\n"
@@ -360,14 +378,14 @@ help_screen_t help_screens[] = {
     "short-sol",
 "The following configurations may produce shorter solutions:\n"
 "\n"
-"    fc-solve -opt\n"
-"    fc-solve --method a-star -opt\n"
-"    fc-solve --reparent-states -opt\n"
-"    fc-solve --method a-star --reparent-states -opt\n"
+"    fc-solve -l children-playing-ball\n"
+"    fc-solve -l gooey-unknown-thing\n"
+"    fc-solve -l slick-rock\n"
 "\n"
-"If \"--method a-star\" is specified you can set the weights with\n"
-"-asw {comma separated list of 5 numeric weights}, which may improve\n"
-"the length of the solution. (refer to the USAGE file for more information)\n"
+"You may also try adding the \"-opt\" and/or \"--reparent-states\" options\n"
+"which may make things a little better.\n"
+"\n"
+"Refer to the file 'USAGE' for more information.\n"
 },
 {
     "summary",
@@ -399,7 +417,7 @@ enum
     EXIT_AND_RETURN_0 = FCS_CMD_LINE_USER
 };
 
-static void print_help_string(char * key)
+static void print_help_string(const char * key)
 {
     int i;
     for(i=0;help_screens[i].key != NULL ; i++)
@@ -413,8 +431,8 @@ static void print_help_string(char * key)
 
 static int cmd_line_callback(
     void * instance,
-    int argc GCC_UNUSED,
-    char * argv[],
+    int argc,
+    const char * argv[],
     int arg,
     int * num_to_skip,
     int * ret,
@@ -462,50 +480,61 @@ static int cmd_line_callback(
             my_iter_handler,   \
             dc    \
             );        \
-        dc->debug_iter_output_on = 1;
+        dc->debug_iter_output_on = TRUE;
 
         set_iter_handler();
     }
     else if ((!strcmp(argv[arg], "-s")) || (!strcmp(argv[arg], "--state-output")))
     {
         set_iter_handler();
-        dc->debug_iter_state_output = 1;
+        dc->debug_iter_state_output = TRUE;
 #undef set_iter_handler
     }
     else if ((!strcmp(argv[arg], "-p")) || (!strcmp(argv[arg], "--parseable-output")))
     {
-        dc->parseable_output = 1;
+        dc->parseable_output = TRUE;
     }
     else if ((!strcmp(argv[arg], "-c")) || (!strcmp(argv[arg], "--canonized-order-output")))
     {
-        dc->canonized_order_output = 1;
+        dc->canonized_order_output = TRUE;
+    }
+    else if ((!strcmp(argv[arg], "-o")) || (!strcmp(argv[arg], "--output")))
+    {
+        arg++;
+        if (arg == argc)
+        {
+            return FCS_CMD_LINE_STOP;
+        }
+        *num_to_skip = 2;
+        dc->output_filename = (const char *)argv[arg];
+        return FCS_CMD_LINE_SKIP;
     }
     else if ((!strcmp(argv[arg], "-t")) || (!strcmp(argv[arg], "--display-10-as-t")))
     {
-        dc->display_10_as_t = 1;
+        dc->display_10_as_t = TRUE;
     }
     else if ((!strcmp(argv[arg], "-m")) || (!strcmp(argv[arg], "--display-moves")))
     {
-        dc->display_moves = 1;
-        dc->display_states = 0;
+        dc->display_moves = TRUE;
+        dc->display_states = FALSE;
     }
     else if ((!strcmp(argv[arg], "-sn")) || (!strcmp(argv[arg], "--standard-notation")))
     {
-        dc->standard_notation = 1;
+        dc->standard_notation = STANDARD_NOTATION_REGULAR;
 
     }
     else if ((!strcmp(argv[arg], "-snx")) || (!strcmp(argv[arg], "--standard-notation-extended")))
     {
-        dc->standard_notation = 2;
+        dc->standard_notation = STANDARD_NOTATION_EXTENDED;
     }
     else if ((!strcmp(argv[arg], "-sam")) || (!strcmp(argv[arg], "--display-states-and-moves")))
     {
-        dc->display_moves = 1;
-        dc->display_states = 1;
+        dc->display_moves = TRUE;
+        dc->display_states = TRUE;
     }
     else if ((!strcmp(argv[arg], "-pi")) || (!strcmp(argv[arg], "--display-parent-iter")))
     {
-        dc->display_parent_iter_num = 1;
+        dc->display_parent_iter_num = TRUE;
     }
     else if ((!strcmp(argv[arg], "--reset")))
     {
@@ -529,7 +558,7 @@ static int cmd_line_callback(
 
 
 static int command_num = 0;
-static int debug_iter_output_on = 0;
+static int debug_iter_output_on = FALSE;
 
 static void select_signal_handler(int signal_num GCC_UNUSED)
 {
@@ -559,7 +588,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
                 NULL,
                 NULL
                 );
-            debug_iter_output_on = 0;
+            debug_iter_output_on = FALSE;
         }
         else
         {
@@ -568,7 +597,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
                 my_iter_handler,
                 dc
                 );
-            debug_iter_output_on = 1;
+            debug_iter_output_on = TRUE;
         }
     }
     else if (command_num == 2)
@@ -580,7 +609,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
 }
 
 
-static char * known_parameters[] = {
+static freecell_solver_str_t known_parameters[] = {
     "-h", "--help",
         "--help-configs", "--help-options", "--help-problems",
         "--help-real-help", "--help-short-sol", "--help-summary",
@@ -594,6 +623,7 @@ static char * known_parameters[] = {
     "-snx", "--standard-notation-extended",
     "-sam", "--display-states-and-moves",
     "-pi", "--display-parent-iter",
+    "-o", "--output",
     "--reset",
     "--version",
     NULL
@@ -627,9 +657,9 @@ int main(int argc, char * argv[])
             freecell_solver_user_cmd_line_parse_args(
                 instance,
                 argc,
-                argv,
+                (freecell_solver_str_t *)argv,
                 1,
-                known_parameters,
+                (freecell_solver_str_t *)known_parameters,
                 cmd_line_callback,
                 &debug_context,
                 &error_string,
@@ -740,19 +770,45 @@ int main(int argc, char * argv[])
         free(error_string);
         error_string = NULL;
     }
+    else if (ret == FCS_STATE_FLARES_PLAN_ERROR)
+    {
+        const char * error_string;
+
+        error_string = freecell_solver_user_get_last_error_string(instance);
+
+        printf("Flares Plan: %s\n", error_string);
+    }
     else
     {
+        FILE * output_fh;
+
+        if (debug_context.output_filename)
+        {
+            output_fh = fopen(debug_context.output_filename, "wt");
+            if (! output_fh)
+            {
+                fprintf(stderr,
+                        "Could not open output file '%s' for writing!",
+                        debug_context.output_filename
+                       );
+                return -1;
+            }
+        }
+        else
+        {
+            output_fh = stdout;
+        }
+
         if (ret == FCS_STATE_WAS_SOLVED)
         {
-            printf("-=-=-=-=-=-=-=-=-=-=-=-\n\n");
+            fprintf(output_fh, "-=-=-=-=-=-=-=-=-=-=-=-\n\n");
             {
                 fcs_move_t move;
                 FILE * move_dump;
                 char * as_string;
                 int move_num = 0;
 
-                move_dump = stdout;
-
+                move_dump = output_fh;
 
                 if (debug_context.display_states)
                 {
@@ -843,23 +899,31 @@ int main(int argc, char * argv[])
                 }
             }
 
-            printf("This game is solveable.\n");
+            fprintf(output_fh, "This game is solveable.\n");
         }
         else
         {
-            printf ("I could not solve this game.\n");
+            fprintf (output_fh, "I could not solve this game.\n");
         }
 
-        printf(
+        fprintf(
+            output_fh,
             "Total number of states checked is %i.\n",
             freecell_solver_user_get_num_times(instance)
             );
 #if 1
-        printf(
+        fprintf(
+            output_fh,
             "This scan generated %i states.\n",
             freecell_solver_user_get_num_states_in_collection(instance)
             );
 #endif
+
+        if (debug_context.output_filename)
+        {
+            fclose(output_fh);
+            output_fh = NULL;
+        }
     }
 
     freecell_solver_user_free(instance);
