@@ -11,6 +11,7 @@ use File::Spec;
 use File::Copy;
 use File::Path;
 use Getopt::Long;
+use Env::Path;
 
 sub run_tests
 {
@@ -19,11 +20,11 @@ sub run_tests
     exec("runprove", @$tests);
 }
 
-my $tests_glob = "*.{exe,t}";
+my $tests_glob = "*.{exe,py,t}";
 
 GetOptions(
     '--glob=s' => \$tests_glob,
-);
+) or die "--glob='tests_glob'";
 
 {
     local $ENV{FCS_PATH} = Cwd::getcwd();
@@ -72,6 +73,25 @@ GetOptions(
 
 
     local $ENV{FREECELL_SOLVER_PRESETRC} = $testing_preset_rc;
+    local $ENV{FREECELL_SOLVER_QUIET} = 1;
+    Env::Path->PATH->Prepend(
+        File::Spec->catdir(Cwd::getcwd(), "board_gen"),
+        File::Spec->catdir(Cwd::getcwd(), "t", "scripts"),
+    );
+
+    local $ENV{HARNESS_ALT_INTRP_FILE} =
+        File::Spec->rel2abs(
+            File::Spec->catdir(
+                File::Spec->curdir(),
+                "t", "config", "alternate-interpreters.yml",
+            ),
+        )
+        ;
+
+    local $ENV{HARNESS_PLUGINS} = 
+        "ColorSummary ColorFileVerdicts AlternateInterpreters"
+        ;
+
     chdir("$FindBin::Bin/t");
     if (system("make", "-s"))
     {
@@ -89,6 +109,11 @@ GetOptions(
         glob("t/$tests_glob")
         ;
 
+    if (! $ENV{FCS_TEST_BUILD})
+    {
+        @tests = grep { !/build-process/ } @tests;
+    }
+
     {
         # local $ENV{FCS_PATH} = dirname(which("fc-solve"));
         print STDERR "FCS_PATH = $ENV{FCS_PATH}\n";
