@@ -61,18 +61,13 @@ char fc_solve_simple_simon_nothing;
     )
 
 
-int fc_solve_sfs_simple_simon_move_sequence_to_founds(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_founds)
 {
     tests_declare_accessors()
 
 
-    fcs_move_t temp_move;
+    fcs_internal_move_t temp_move;
 
-    int check;
 
     /*
      * stack - the stack index from which to move cards to the founds.
@@ -89,12 +84,12 @@ int fc_solve_sfs_simple_simon_move_sequence_to_founds(
     fcs_card_t card, above_card;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
 
     temp_move = fc_solve_empty_move;
@@ -141,9 +136,9 @@ int fc_solve_sfs_simple_simon_move_sequence_to_founds(
                 }
 
 
-                fcs_move_set_type(temp_move, FCS_MOVE_TYPE_SEQ_TO_FOUNDATION);
-                fcs_move_set_src_stack(temp_move, stack_idx);
-                fcs_move_set_foundation(temp_move,suit);
+                fcs_int_move_set_type(temp_move, FCS_MOVE_TYPE_SEQ_TO_FOUNDATION);
+                fcs_int_move_set_src_stack(temp_move, stack_idx);
+                fcs_int_move_set_foundation(temp_move,suit);
                 fcs_move_stack_push(moves,temp_move);
 
                 sfs_check_state_end();
@@ -151,22 +146,17 @@ int fc_solve_sfs_simple_simon_move_sequence_to_founds(
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
 
-int fc_solve_sfs_simple_simon_move_sequence_to_true_parent(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_true_parent)
 {
     tests_declare_accessors()
 
 
 
-    int check;
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     /*
      * stack - the source stack index on which the sequence currently resides.
@@ -197,12 +187,12 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent(
     fcs_cards_column_t col, dest_col;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
 
     num_vacant_stacks = soft_thread->num_vacant_stacks;
@@ -241,7 +231,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent(
                         {
                             /* This is a suitable parent - let's check if we
                              * have enough empty stacks to make the move feasible */
-                            if (calc_max_sequence_move(0, num_vacant_stacks) >= num_true_seqs)
+                            if (calc_max_simple_simon_seq_move(num_vacant_stacks) >= num_true_seqs)
                             {
                                 /* We can do it - so let's move */
 
@@ -281,20 +271,15 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent(
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent)
 {
     tests_declare_accessors()
 
 
 
-    int check;
 
     /*
      * stack - the source stack index
@@ -315,15 +300,15 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
     fcs_cards_column_t col, dest_col;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
 
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
 
     num_vacant_stacks = soft_thread->num_vacant_stacks;
@@ -375,7 +360,7 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
                     {
                         /* This is a suitable parent - let's check if we
                          * have enough empty stacks to make the move feasible */
-                        if (calc_max_sequence_move(0, num_vacant_stacks) >= num_true_seqs)
+                        if (calc_max_simple_simon_seq_move(num_vacant_stacks) >= num_true_seqs)
                         {
                             /* We can do it - so let's move */
 
@@ -396,19 +381,14 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent(
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above)
 {
     tests_declare_accessors()
 
 
-    int check;
     fcs_cards_column_t col;
     fcs_cards_column_t clear_junk_dest_col;
     fcs_cards_column_t dest_col;
@@ -453,14 +433,14 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above
     int junk_move_to_stacks[MAX_NUM_STACKS];
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
     num_vacant_stacks = soft_thread->num_vacant_stacks;
 
@@ -567,7 +547,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above
                                             clear_junk_dest_card = fcs_col_get_card(clear_junk_dest_col, clear_junk_stack_len-1);
                                             if (fcs_is_ss_false_parent(clear_junk_dest_card, fcs_col_get_card(dest_col, seq_points[false_seq_index])))
                                             {
-                                                if (calc_max_sequence_move(0, after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
+                                                if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
                                                 {
                                                     stacks_map[clear_junk_dest_stack] = 1;
                                                     break;
@@ -586,7 +566,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above
                                         /* Check if there is a vacant stack */
                                         if (num_vacant_stacks > 0)
                                         {
-                                            if (calc_max_sequence_move(0, after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
+                                            if (calc_max_simple_simon_seq_move(after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
                                             {
                                                 /* Find an empty stack and designate it as the destination for the junk */
                                                 for(
@@ -616,7 +596,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above
 
                                 if (false_seq_index == num_separate_false_seqs)
                                 {
-                                    if (calc_max_sequence_move(0, after_junk_num_freestacks) >= num_true_seqs)
+                                    if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= num_true_seqs)
                                     {
                                         /*
                                          * We can do it - so let's move everything.
@@ -682,21 +662,16 @@ int fc_solve_sfs_simple_simon_move_sequence_to_true_parent_with_some_cards_above
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent)
 {
     tests_declare_accessors()
 
     int stack_idx, cards_num, suit, a;
     fcs_card_t card, dest_card;
     int card_num, num_true_seqs, ds, dest_cards_num ;
-    int check;
     int src_card_height, num_separate_false_seqs, above_num_true_seqs[MAX_NUM_CARDS_IN_A_STACK];
     int seq_points[MAX_NUM_CARDS_IN_A_STACK];
     int stacks_map[MAX_NUM_STACKS];
@@ -705,9 +680,9 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
     int junk_move_to_stacks[MAX_NUM_CARDS_IN_A_STACK];
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     fcs_cards_column_t col;
     fcs_cards_column_t dest_col;
@@ -715,7 +690,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
 
     num_vacant_stacks = soft_thread->num_vacant_stacks;
@@ -833,7 +808,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
                                         clear_junk_dest_card = fcs_col_get_card(clear_junk_dest_col, clear_junk_stack_len-1);
                                         if (fcs_is_ss_false_parent(clear_junk_dest_card, fcs_col_get_card(col, seq_points[false_seq_index])))
                                         {
-                                            if (calc_max_sequence_move(0, after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
+                                            if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
                                             {
                                                 stacks_map[clear_junk_dest_stack] = 1;
                                                 break;
@@ -852,7 +827,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
                                     /* Check if there is a vacant stack */
                                     if (num_vacant_stacks > 0)
                                     {
-                                        if (calc_max_sequence_move(0, after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
+                                        if (calc_max_simple_simon_seq_move(after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
                                         {
                                             /* Find an empty stack and designate it as the destination for the junk */
                                             for(
@@ -882,7 +857,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
 
                             if (false_seq_index == num_separate_false_seqs)
                             {
-                                if (calc_max_sequence_move(0, after_junk_num_freestacks) > num_true_seqs)
+                                if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) > num_true_seqs)
                                 {
                                     sfs_check_state_begin();
 
@@ -928,18 +903,13 @@ int fc_solve_sfs_simple_simon_move_sequence_with_some_cards_above_to_true_parent
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_with_some_cards_above(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_with_some_cards_above)
 {
     tests_declare_accessors()
 
-    int check;
 
     /*
      * stack - the source stack index
@@ -982,9 +952,9 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
     int num_true_seqs;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     fcs_cards_column_t col;
     fcs_cards_column_t dest_col;
@@ -992,7 +962,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
     num_vacant_stacks = soft_thread->num_vacant_stacks;
 
@@ -1125,7 +1095,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
                                                 (fcs_col_get_card(col, end_of_junk+1)) :
                                                 (fcs_col_get_card(dest_col, seq_points[false_seq_index]))
                                         )
-                                    )   
+                                    )
                                     ;
 
                                     int the_num_true_seqs =
@@ -1152,7 +1122,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
                                             clear_junk_dest_card = fcs_col_get_card(clear_junk_dest_col, clear_junk_stack_len-1);
                                             if (fcs_is_ss_false_parent(clear_junk_dest_card, the_card))
                                             {
-                                                if (calc_max_sequence_move(0, after_junk_num_freestacks) >= the_num_true_seqs)
+                                                if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= the_num_true_seqs)
                                                 {
                                                     stacks_map[clear_junk_dest_stack] = 1;
                                                     break;
@@ -1171,7 +1141,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
                                         /* Check if there is a vacant stack */
                                         if (num_vacant_stacks > 0)
                                         {
-                                            if (calc_max_sequence_move(0, after_junk_num_freestacks-1) >= the_num_true_seqs)
+                                            if (calc_max_simple_simon_seq_move(after_junk_num_freestacks-1) >= the_num_true_seqs)
                                             {
                                                 /* Find an empty stack and designate it as the destination for the junk */
                                                 for(
@@ -1201,7 +1171,7 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
 
                                 if (false_seq_index == num_separate_false_seqs+1)
                                 {
-                                    if (calc_max_sequence_move(0, after_junk_num_freestacks) >= num_true_seqs)
+                                    if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= num_true_seqs)
                                     {
                                         /* We can do it - so let's move everything */
 
@@ -1257,18 +1227,13 @@ int fc_solve_sfs_simple_simon_move_sequence_with_junk_seq_above_to_true_parent_w
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_some_cards_above(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_some_cards_above)
 {
     tests_declare_accessors()
 
-    int check;
 
     /*
      * stack - the source stack index
@@ -1308,10 +1273,10 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_som
     int junk_move_to_stacks[MAX_NUM_STACKS];
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
 
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     fcs_cards_column_t col;
     fcs_cards_column_t dest_col;
@@ -1319,7 +1284,7 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_som
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
     num_vacant_stacks = soft_thread->num_vacant_stacks;
 
@@ -1434,7 +1399,7 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_som
                                             clear_junk_dest_card = fcs_col_get_card(clear_junk_dest_col, clear_junk_stack_len-1);
                                             if (fcs_is_ss_false_parent(clear_junk_dest_card, the_card))
                                             {
-                                                if (calc_max_sequence_move(0, after_junk_num_freestacks) >= the_num_true_seqs)
+                                                if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= the_num_true_seqs)
                                                 {
                                                     stacks_map[clear_junk_dest_stack] = 1;
                                                     break;
@@ -1459,7 +1424,7 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_som
                                 {
                                     /* This is a suitable parent - let's check if we
                                      * have enough empty stacks to make the move feasible */
-                                    if (calc_max_sequence_move(0, num_vacant_stacks) >= num_true_seqs)
+                                    if (calc_max_simple_simon_seq_move(num_vacant_stacks) >= num_true_seqs)
                                     {
                                         /* We can do it - so let's move */
 
@@ -1507,18 +1472,13 @@ int fc_solve_sfs_simple_simon_move_whole_stack_sequence_to_false_parent_with_som
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack)
 {
     tests_declare_accessors()
 
-    int check;
 
     int stack_idx, cards_num, parent_card_height, child_card_height;
     fcs_card_t parent_card;
@@ -1528,16 +1488,16 @@ int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
     int child_seq_index;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
     fcs_cards_column_t col;
     fcs_cards_column_t clear_junk_dest_col;
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
+    SET_GAME_PARAMS();
 #endif
     num_vacant_stacks = soft_thread->num_vacant_stacks;
 
@@ -1696,7 +1656,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                     clear_junk_dest_card = fcs_col_get_card(clear_junk_dest_col, clear_junk_stack_len-1);
                                     if (fcs_is_ss_false_parent(clear_junk_dest_card, fcs_col_get_card(col, seq_points[false_seq_index])))
                                     {
-                                        if (calc_max_sequence_move(0, after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
+                                        if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= above_num_true_seqs[false_seq_index])
                                         {
                                             stacks_map[clear_junk_dest_stack] = 1;
                                             break;
@@ -1715,7 +1675,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                                 /* Check if there is a vacant stack */
                                 if (num_vacant_stacks > 0)
                                 {
-                                    if (calc_max_sequence_move(0, after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
+                                    if (calc_max_simple_simon_seq_move(after_junk_num_freestacks-1) >= above_num_true_seqs[false_seq_index])
                                     {
                                         /* Find an empty stack and designate it as the destination for the junk */
                                         for(
@@ -1746,7 +1706,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
                         if (false_seq_index == num_separate_false_seqs)
                         {
                             /* Let's check if we can move the child after we are done moving all the junk cards */
-                            if (calc_max_sequence_move(0, after_junk_num_freestacks) >= child_num_true_seqs)
+                            if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >= child_num_true_seqs)
                             {
                                 /* We can do it - so let's move everything */
 
@@ -1793,19 +1753,13 @@ int fc_solve_sfs_simple_simon_move_sequence_to_parent_on_the_same_stack(
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
-int fc_solve_sfs_simple_simon_move_sequence_to_false_parent(
-        fc_solve_soft_thread_t * soft_thread,
-        fcs_state_extra_info_t * ptr_state_val,
-        fcs_derived_states_list_t * derived_states_list
-        )
+DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_false_parent)
 {
     tests_declare_accessors();
 
-    int check;
-
     /*
      * stack - the source stack index
      * cards_num - number of cards in "stack"
@@ -1822,17 +1776,17 @@ int fc_solve_sfs_simple_simon_move_sequence_to_false_parent(
     int num_true_seqs, seq_size, h, ds, dest_cards_num;
     fcs_cards_column_t col, dest_col;
     int cards_num;
-    int num_vacant_stacks;
+    fcs_game_limit_t num_vacant_stacks;
 
 #ifndef HARD_CODED_NUM_STACKS
-    int stacks_num;
+    DECLARE_GAME_PARAMS();
 #endif
 
     tests_define_accessors();
 
 #ifndef HARD_CODED_NUM_STACKS
-    stacks_num = instance->stacks_num;
-#endif    
+    SET_GAME_PARAMS();
+#endif
     num_vacant_stacks = soft_thread->num_vacant_stacks;
 
     for(stack_idx=0 ; stack_idx < LOCAL_STACKS_NUM ; stack_idx++)
@@ -1885,7 +1839,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_false_parent(
 
 			/* This is a suitable parent - let's check if we
              * have enough empty stacks to make the move feasible */
-            if (calc_max_sequence_move(0, num_vacant_stacks) < num_true_seqs)
+            if (calc_max_simple_simon_seq_move(num_vacant_stacks) < num_true_seqs)
             {
 				continue;
 			}
@@ -1901,7 +1855,7 @@ int fc_solve_sfs_simple_simon_move_sequence_to_false_parent(
         }
     }
 
-    return FCS_STATE_IS_NOT_SOLVEABLE;
+    return;
 }
 
 #undef state
