@@ -39,9 +39,12 @@ extern "C" {
 #include "fcs_move.h"
 
 #include "inline.h"
+#include "bool.h"
 
 #include "card.h"
 
+#include "internal_move_struct.h"
+
 #if MAX_NUM_INITIAL_CARDS_IN_A_STACK+12>(MAX_NUM_DECKS*52)
 #define MAX_NUM_CARDS_IN_A_STACK (MAX_NUM_DECKS*52)
 #else
@@ -51,8 +54,8 @@ extern "C" {
 #define MAX_NUM_SCANS_BUCKETS 1
 #define MAX_NUM_SCANS (MAX_NUM_SCANS_BUCKETS * (sizeof(int)*8))
 
-#define is_scan_visited(ptr_state_val, scan_id) (ptr_state_val->scan_visited[(scan_id)>>FCS_INT_BIT_SIZE_LOG2] & (1 << ((scan_id)&((1<<(FCS_INT_BIT_SIZE_LOG2))-1))))
-#define set_scan_visited(ptr_state_val, scan_id) { ptr_state_val->scan_visited[(scan_id)>>FCS_INT_BIT_SIZE_LOG2] |= (1 << ((scan_id)&((1<<(FCS_INT_BIT_SIZE_LOG2))-1))); }
+#define is_scan_visited(ptr_state, scan_id) ((FCS_S_SCAN_VISITED(ptr_state))[(scan_id)>>FCS_INT_BIT_SIZE_LOG2] & (1 << ((scan_id)&((1<<(FCS_INT_BIT_SIZE_LOG2))-1))))
+#define set_scan_visited(ptr_state, scan_id) { (FCS_S_SCAN_VISITED(ptr_state))[(scan_id)>>FCS_INT_BIT_SIZE_LOG2] |= (1 << ((scan_id)&((1<<(FCS_INT_BIT_SIZE_LOG2))-1))); }
 
 
 #ifdef DEBUG_STATES
@@ -259,22 +262,22 @@ typedef struct fcs_struct_state_t fcs_state_t;
     ((state).talon[pos])
 #endif
 
-#define fcs_copy_stack(state_key, state_val, idx, buffer) \
+#define fcs_copy_stack(state_struct, idx, buffer) \
     {     \
-        if (! ((state_val).stacks_copy_on_write_flags & (1 << idx)))        \
+        if (! ((state_struct).info.stacks_copy_on_write_flags & (1 << idx)))        \
         {          \
             fcs_cards_column_t copy_stack_col; \
                                     \
-            (state_val).stacks_copy_on_write_flags |= (1 << idx);       \
-            copy_stack_col = fcs_state_get_col(state_key, idx); \
+            (state_struct).info.stacks_copy_on_write_flags |= (1 << idx);       \
+            copy_stack_col = fcs_state_get_col(((state_struct).s), idx); \
             memcpy(&buffer[idx << 7], copy_stack_col, fcs_col_len(copy_stack_col)+1); \
-            fcs_state_get_col(state_key, idx) = &buffer[idx << 7];     \
+            fcs_state_get_col((state_struct.s), idx) = &buffer[idx << 7];     \
         }     \
     }
 
-#define fcs_duplicate_state_extra(ptr_dest_key, ptr_dest_val, ptr_src_key, ptr_src_val)  \
+#define fcs_duplicate_state_extra(ptr_dest, ptr_src)  \
     {   \
-        (ptr_dest_val)->stacks_copy_on_write_flags = 0; \
+        (ptr_dest)->info.stacks_copy_on_write_flags = 0; \
     }
 
 typedef char fcs_locs_t;
@@ -349,20 +352,27 @@ typedef char fcs_locs_t;
 #define fcs_col_push_col_card(dest_col, src_col, card_idx) \
     fcs_col_push_card((dest_col), fcs_col_get_card((src_col), (card_idx)))
 
+#ifdef FCS_RCS_STATES
 #define fcs_duplicate_state(ptr_dest_key, ptr_dest_val, ptr_src_key, ptr_src_val) \
     { \
     *(ptr_dest_key) = *(ptr_src_key); \
     *(ptr_dest_val) = *(ptr_src_val); \
-    (ptr_dest_val)->key = ptr_dest_key; \
-    fcs_duplicate_state_extra(ptr_dest_key, ptr_dest_val, ptr_src_key, ptr_src_val);   \
+    fcs_duplicate_state_extra(ptr_dest, ptr_src);   \
     }
+#else
+#define fcs_duplicate_state(ptr_dest, ptr_src) \
+    { \
+    *(ptr_dest) = *(ptr_src); \
+    fcs_duplicate_state_extra(ptr_dest, ptr_src);   \
+    }
+#endif
 
 #if defined(COMPACT_STATES) || defined(DEBUG_STATES)
 
-#define fcs_duplicate_state_extra(ptr_dest_key, ptr_dest_val, ptr_src_key, ptr_src_val) \
-    {} 
+#define fcs_duplicate_state_extra(ptr_dest, ptr_src) \
+    {}
 
-#define fcs_copy_stack(state_key, state_val, idx, buffer) {}
+#define fcs_copy_stack(state_struct, idx, buffer) {}
 
 #endif
 
@@ -377,7 +387,7 @@ typedef char fcs_locs_t;
     (fcs_card_set_flipped(fcs_col_get_card((col), (c)), 0))
 #endif
 
-/* These are macros that are common to COMPACT_STATES and 
+/* These are macros that are common to COMPACT_STATES and
  * INDIRECT_STACK_STATES */
 #if defined(COMPACT_STATES) || defined(INDIRECT_STACK_STATES)
 
@@ -411,14 +421,76 @@ typedef char fcs_locs_t;
 
 #endif
 
+
+typedef unsigned char fcs_game_limit_t;
+
+struct fcs_state_keyval_pair_struct;
+
+/*
+ * NOTE: the order of elements here is intended to reduce framgmentation
+ * and memory consumption. Namely:
+ *
+ * 1. Pointers come first.
+ *
+ * 2. ints (32-bit on most platform) come next.
+ *
+ * 3. chars come next.
+ *
+ * */
 struct fcs_state_extra_info_struct
 {
+#ifdef FCS_RCS_STATES
+    struct fcs_state_extra_info_struct * parent;
+#else
+    struct fcs_state_keyval_pair_struct * parent;
+#endif
+    fcs_move_stack_t * moves_to_parent;
+
+#ifndef FCS_WITHOUT_DEPTH_FIELD
+    int depth;
+#endif
+
+#ifndef FCS_WITHOUT_VISITED_ITER
+    /*
+     * The iteration in which this state was marked as visited
+     * */
+    int visited_iter;
+#endif
+
+
+    /*
+     * This is a vector of flags - one for each scan. Each indicates whether
+     * its scan has already visited this state
+     * */
+    int scan_visited[MAX_NUM_SCANS_BUCKETS];
+
+#ifdef INDIRECT_STACK_STATES
+    /*
+     * A vector of flags that indicates which stacks were already copied.
+     * */
+    int stacks_copy_on_write_flags;
+#endif
+
+    /*
+     * This is the number of direct children of this state which were not
+     * yet declared as dead ends. Once this counter reaches zero, this
+     * state too is declared as a dead end.
+     *
+     * It was converted to an unsigned short , because it is extremely
+     * unlikely that a state will have more than 64K active children.
+     * */
+    unsigned short num_active_children;
+
+#ifndef FCS_WITHOUT_LOCS_FIELDS
+    /*
+     * These contain the location of the original stacks and freecells
+     * in the permutation of them. They are sorted by the canonization
+     * function.
+     * */
     fcs_locs_t stack_locs[MAX_NUM_STACKS];
     fcs_locs_t fc_locs[MAX_NUM_FREECELLS];
-    fcs_state_t * key;
-    struct fcs_state_extra_info_struct * parent_val;
-    fcs_move_stack_t * moves_to_parent;
-    int depth;
+#endif
+
     /*
      * This field contains global, scan-independant flags, which are used
      * from the FCS_VISITED_* enum below.
@@ -431,42 +503,90 @@ struct fcs_state_extra_info_struct
      *
      * FCS_VISITED_DEAD_END - indicates that the state does not lead to
      * anywhere useful, and scans should not examine it in the first place.
+     *
+     * FCS_VISITED_GENERATED_BY_PRUNING - indicates that the state was
+     * generated by pruning, so one can skip calling the pruning function
+     * for it.
      * */
-    int visited;
-    /*
-     * The iteration in which this state was marked as visited
-     * */
-    int visited_iter;
-    /*
-     * This is the number of direct children of this state which were not
-     * yet declared as dead ends. Once this counter reaches zero, this
-     * state too is declared as a dead end.
-     * */
-    int num_active_children;
-    /*
-     * This is a vector of flags - one for each scan. Each indicates whether
-     * its scan has already visited this state
-     * */
-    int scan_visited[MAX_NUM_SCANS_BUCKETS];
-#ifdef INDIRECT_STACK_STATES
-    /*
-     * A vector of flags that indicates which stacks were already copied.
-     * */
-    int stacks_copy_on_write_flags;
-#endif
+    fcs_game_limit_t visited;
+
 };
 
+#ifdef FCS_WITHOUT_LOCS_FIELDS
+typedef struct
+{
+    /*
+     * These contain the location of the original stacks and freecells
+     * in the permutation of them. They are sorted by the canonization
+     * function.
+     * */
+    fcs_locs_t stack_locs[MAX_NUM_STACKS];
+    fcs_locs_t fc_locs[MAX_NUM_FREECELLS];
+} fcs_state_locs_struct_t;
+#endif
+
 typedef struct fcs_state_extra_info_struct fcs_state_extra_info_t;
 
-typedef struct
+struct fcs_state_keyval_pair_struct
 {
-    fcs_state_t s;
-    fcs_state_extra_info_t info;
-} fcs_state_keyval_pair_t;
+    union
+    {
+        struct
+        {
+            fcs_state_t s;
+            fcs_state_extra_info_t info;
+        };
+        struct fcs_state_keyval_pair_struct * next;
+    };
+};
+
+typedef struct fcs_state_keyval_pair_struct fcs_state_keyval_pair_t;
+
+/*
+ * This type is the struct that is collectible inside the hash.
+ *
+ * In FCS_RCS_STATES we only collect the extra_info's and the state themselves
+ * are kept in an LRU cache because they can be calculated from the
+ * extra_infos and the original state by applying the moves.
+ *
+ * */
+#ifdef FCS_RCS_STATES
+typedef fcs_state_extra_info_t fcs_collectible_state_t;
+#define FCS_S_ACCESSOR(s, field) ((s)->field)
+#define FCS_S_NEXT(s) FCS_S_ACCESSOR(s, parent)
+#else
+typedef fcs_state_keyval_pair_t fcs_collectible_state_t;
+#define FCS_S_ACCESSOR(s, field) (((s)->info).field)
+#define FCS_S_NEXT(s) ((s)->next)
+#endif
+
+#define FCS_S_PARENT(s) FCS_S_ACCESSOR(s, parent)
+#define FCS_S_NUM_ACTIVE_CHILDREN(s) FCS_S_ACCESSOR(s, num_active_children)
+#define FCS_S_MOVES_TO_PARENT(s) FCS_S_ACCESSOR(s, moves_to_parent)
+#define FCS_S_VISITED(s) FCS_S_ACCESSOR(s, visited)
+
+#ifndef FCS_WITHOUT_LOCS_FIELDS
+#define FCS_S_STACK_LOCS(s) FCS_S_ACCESSOR(s, stack_locs)
+#define FCS_S_FC_LOCS(s) FCS_S_ACCESSOR(s, fc_locs)
+#endif
+
+#define FCS_S_SCAN_VISITED(s) FCS_S_ACCESSOR(s, scan_visited)
+
+#ifndef FCS_WITHOUT_DEPTH_FIELD
+#define FCS_S_DEPTH(s) FCS_S_ACCESSOR(s, depth)
+#endif
+
+#ifndef FCS_WITHOUT_VISITED_ITER
+#define FCS_S_VISITED_ITER(s) FCS_S_ACCESSOR(s, visited_iter)
+#endif
 
 typedef struct {
     fcs_state_t * key;
     fcs_state_extra_info_t * val;
+    fcs_collectible_state_t * s;
+#ifdef FCS_WITHOUT_LOCS_FIELDS
+    fcs_state_locs_struct_t locs;
+#endif
 } fcs_standalone_state_ptrs_t;
 
 extern fcs_card_t fc_solve_empty_card;
@@ -501,13 +621,28 @@ extern fcs_card_t fc_solve_empty_card;
     ((state).talon[(int)((fcs_klondike_talon_stack_pos(state)--)+1)] = fc_solve_empty_card)
 #endif
 
-
 extern void fc_solve_canonize_state(
-    fcs_state_extra_info_t * state_val,
+#ifdef FCS_RCS_STATES
+    fcs_state_t * state_key,
+#endif
+    fcs_collectible_state_t * state,
     int freecells_num,
     int stacks_num
     );
 
+#ifdef FCS_WITHOUT_LOCS_FIELDS
+void fc_solve_canonize_state_with_locs(
+#ifdef FCS_RCS_STATES
+    fcs_state_t * state_key,
+    fcs_collectible_state_t * state_val,
+#else
+    fcs_collectible_state_t * state,
+#endif
+    fcs_state_locs_struct_t * locs,
+    int freecells_num,
+    int stacks_num);
+#endif
+
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
 
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_LIBREDBLACK_TREE)
@@ -516,12 +651,17 @@ typedef void * fcs_compare_context_t;
 typedef const void * fcs_compare_context_t;
 #endif
 
-extern int fc_solve_state_compare(const void * s1, const void * s2);
+#if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
+static GCC_INLINE int fc_solve_state_compare(const void * s1, const void * s2)
+{
+    return memcmp(s1,s2,sizeof(fcs_state_t));
+}
+#endif
+
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
 extern int fc_solve_state_compare_equal(const void * s1, const void * s2);
 #endif
 extern int fc_solve_state_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
-extern int fc_solve_state_extra_info_compare_with_context(const void * s1, const void * s2, fcs_compare_context_t context);
 
 #else
 extern int fc_solve_state_compare_indirect(const void * s1, const void * s2);
@@ -591,47 +731,59 @@ extern int fc_solve_u2p_card_number(const char * string);
 extern int fc_solve_u2p_suit(const char * deck);
 
 static GCC_INLINE void fc_solve_state_init(
-    fcs_state_t * state_key,
-    fcs_state_extra_info_t * state_val,
+    fcs_state_keyval_pair_t * state,
     int stacks_num
 #ifdef INDIRECT_STACK_STATES
     ,char * indirect_stacks_buffer
 #endif
     )
 {
+#if ((!defined(FCS_WITHOUT_CARD_FLIPPING)) || (!defined(FCS_WITHOUT_LOCS_FIELDS)) || defined(INDIRECT_STACK_STATES))
     int i;
+#endif
 
-    memset(state_key, 0, sizeof(*state_key));
 
-    for(i=0;i<MAX_NUM_STACKS;i++)
+    memset(&(state->s), 0, sizeof(state->s));
+
+#ifndef FCS_WITHOUT_LOCS_FIELDS
     {
-        state_val->stack_locs[i] = (char)i;
+        fcs_locs_t * stack_locs = state->info.stack_locs;
+        for ( i=0 ; i<MAX_NUM_STACKS ; i++)
+        {
+            stack_locs[i] = (fcs_locs_t)i;
+        }
     }
+#endif
 #ifdef INDIRECT_STACK_STATES
     for(i=0;i<stacks_num;i++)
     {
-        state_key->stacks[i] = &indirect_stacks_buffer[i << 7];
-        memset(state_key->stacks[i], '\0', MAX_NUM_DECKS*52+1);
+        state->s.stacks[i] = &indirect_stacks_buffer[i << 7];
+        memset(state->s.stacks[i], '\0', MAX_NUM_DECKS*52+1);
     }
     for(;i<MAX_NUM_STACKS;i++)
     {
-        state_key->stacks[i] = NULL;
+        state->s.stacks[i] = NULL;
     }
 #endif
-    for(i=0;i<MAX_NUM_FREECELLS;i++)
+#ifndef FCS_WITHOUT_LOCS_FIELDS
+    for ( i=0 ; i<MAX_NUM_FREECELLS ; i++)
     {
-        state_val->fc_locs[i] = (char)i;
+        state->info.fc_locs[i] = (char)i;
     }
-    state_val->key = state_key;
-    state_val->parent_val = NULL;
-    state_val->moves_to_parent = NULL;
-    state_val->depth = 0;
-    state_val->visited = 0;
-    state_val->visited_iter = 0;
-    state_val->num_active_children = 0;
-    memset(state_val->scan_visited, '\0', sizeof(state_val->scan_visited));
+#endif
+    state->info.parent = NULL;
+    state->info.moves_to_parent = NULL;
+#ifndef FCS_WITHOUT_DEPTH_FIELD
+    state->info.depth = 0;
+#endif
+    state->info.visited = 0;
+#ifndef FCS_WITHOUT_VISITED_ITER
+    state->info.visited_iter = 0;
+#endif
+    state->info.num_active_children = 0;
+    memset(state->info.scan_visited, '\0', sizeof(state->info.scan_visited));
 #ifdef INDIRECT_STACK_STATES
-    state_val->stacks_copy_on_write_flags = 0;
+    state->info.stacks_copy_on_write_flags = 0;
 #endif
 }
 
@@ -650,8 +802,7 @@ static const char * const fc_solve_num_redeals_prefixes[] = { "Num-Redeals:", "R
 
 static GCC_INLINE int fc_solve_initial_user_state_to_c(
     const char * string,
-    fcs_state_t * out_state_key,
-    fcs_state_extra_info_t * out_state_val,
+    fcs_state_keyval_pair_t * out_state,
     int freecells_num,
     int stacks_num,
     int decks_num
@@ -675,8 +826,7 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c(
     int decks_index[4];
 
     fc_solve_state_init(
-        out_state_key,
-        out_state_val,
+        out_state,
         stacks_num
 #ifdef INDIRECT_STACK_STATES
         , indirect_stacks_buffer
@@ -686,7 +836,7 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c(
 
     first_line = 1;
 
-#define ret (*out_state_key)
+#define ret (out_state->s)
 /* Handle the end of string - shouldn't happen */
 #define handle_eos() \
     { \
@@ -997,13 +1147,19 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c(
 #undef handle_eos
 
 extern char * fc_solve_state_as_string(
-    fcs_state_extra_info_t * state_val,
+#ifdef FCS_RCS_STATES
+    fcs_state_t * key,
+#endif
+    fcs_collectible_state_t * state_pair,
+#ifdef FCS_WITHOUT_LOCS_FIELDS
+    fcs_state_locs_struct_t * state_locs,
+#endif
     int freecells_num,
     int stacks_num,
     int decks_num,
-    int parseable_output,
-    int canonized_order_output,
-    int display_10_as_t
+    fcs_bool_t parseable_output,
+    fcs_bool_t canonized_order_output,
+    fcs_bool_t display_10_as_t
     );
 
 enum
@@ -1016,7 +1172,7 @@ enum
 };
 
 static GCC_INLINE int fc_solve_check_state_validity(
-    fcs_state_extra_info_t * state_val,
+    fcs_state_keyval_pair_t * state_pair,
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -1033,7 +1189,7 @@ static GCC_INLINE int fc_solve_check_state_validity(
     fcs_cards_column_t col;
     fcs_card_t card;
 
-    state = state_val->key;
+    state = &(state_pair->s);
 
     /* Initialize all cards to 0 */
     for(d=0;d<4;d++)
@@ -1113,7 +1269,7 @@ static GCC_INLINE int fc_solve_check_state_validity(
                 *misplaced_card = fc_solve_empty_card;
                 fcs_card_set_suit(*misplaced_card, d);
                 fcs_card_set_num(*misplaced_card, c);
-                return ((cards[d][c] < decks_num) 
+                return ((cards[d][c] < decks_num)
                     ? FCS_STATE_VALIDITY__MISSING_CARD
                     : FCS_STATE_VALIDITY__EXTRA_CARD
                     )
@@ -1137,10 +1293,82 @@ enum
     FCS_VISITED_IN_OPTIMIZED_PATH = 0x2,
     FCS_VISITED_DEAD_END = 0x4,
     FCS_VISITED_ALL_TESTS_DONE = 0x8,
+    FCS_VISITED_GENERATED_BY_PRUNING = 0x10,
 };
 
+#ifndef min
+#define min(a,b) ((a)<(b)?(a):(b))
+#endif
+
+static GCC_INLINE int fc_solve_card_compare(
+        const fcs_card_t * c1,
+        const fcs_card_t * c2
+        )
+{
+    if (fcs_card_card_num(*c1) > fcs_card_card_num(*c2))
+    {
+        return 1;
+    }
+    else if (fcs_card_card_num(*c1) < fcs_card_card_num(*c2))
+    {
+        return -1;
+    }
+    else
+    {
+        if (fcs_card_suit(*c1) > fcs_card_suit(*c2))
+        {
+            return 1;
+        }
+        else if (fcs_card_suit(*c1) < fcs_card_suit(*c2))
+        {
+            return -1;
+        }
+        else
+        {
+            return 0;
+        }
+    }
+}
+
 #if defined(INDIRECT_STACK_STATES)
-extern int fc_solve_stack_compare_for_comparison(const void * v_s1, const void * v_s2);
+static GCC_INLINE int fc_solve_stack_compare_for_comparison(const void * v_s1, const void * v_s2)
+{
+    const fcs_card_t * s1 = (const fcs_card_t *)v_s1;
+    const fcs_card_t * s2 = (const fcs_card_t *)v_s2;
+
+    int min_len;
+    int a, ret;
+
+    min_len = min(s1[0], s2[0]);
+
+    for(a=0;a<min_len;a++)
+    {
+        ret = fc_solve_card_compare(s1+a+1,s2+a+1);
+        if (ret != 0)
+        {
+            return ret;
+        }
+    }
+    /*
+     * The reason I do the stack length comparisons after the card-by-card
+     * comparison is to maintain correspondence with
+     * fcs_stack_compare_for_stack_sort, and with the one card comparison
+     * of the other state representation mechanisms.
+     * */
+    if (s1[0] < s2[0])
+    {
+        return -1;
+    }
+    else if (s1[0] > s2[0])
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;
+    }
+}
+
 #endif
 
 #endif /* FC_SOLVE__STATE_H */
