@@ -25,7 +25,7 @@
  * test_multi_parallel.c - the Freecell Solver range solver.
  *
  * Also see:
- *  - fc_pro_range_solver.c - range solver that can count the moves in 
+ *  - fc_pro_range_solver.c - range solver that can count the moves in
  *  the solution and display the FC-Pro ones.
  *  - threaded_range_solver.c - solve different boards in several POSIX
  *  threads.
@@ -44,21 +44,22 @@
 #include "fcs_cl.h"
 #include "unused.h"
 #include "inline.h"
+#include "bool.h"
 #include "range_solvers_gen_ms_boards.h"
 
 struct fc_solve_display_information_context_struct
 {
-    int debug_iter_state_output;
+    fcs_bool_t debug_iter_state_output;
     int freecells_num;
     int stacks_num;
     int decks_num;
-    int parseable_output;
-    int canonized_order_output;
-    int display_10_as_t;
-    int display_parent_iter_num;
-    int debug_iter_output_on;
-    int display_moves;
-    int display_states;
+    fcs_bool_t parseable_output;
+    fcs_bool_t canonized_order_output;
+    fcs_bool_t display_10_as_t;
+    fcs_bool_t display_parent_iter_num;
+    fcs_bool_t debug_iter_output_on;
+    fcs_bool_t display_moves;
+    fcs_bool_t display_states;
     int standard_notation;
 };
 
@@ -113,7 +114,7 @@ typedef struct pack_item_struct pack_item_t;
 static int cmd_line_callback(
     void * instance,
     int argc GCC_UNUSED,
-    char * argv[],
+    freecell_solver_str_t argv[],
     int arg,
     int * num_to_skip,
     int * ret GCC_UNUSED,
@@ -134,41 +135,41 @@ static int cmd_line_callback(
             my_iter_handler,
             dc
             );
-        dc->debug_iter_output_on = 1;
+        dc->debug_iter_output_on = TRUE;
     }
     else if ((!strcmp(argv[arg], "-s")) || (!strcmp(argv[arg], "--state-output")))
     {
-        dc->debug_iter_state_output = 1;
+        dc->debug_iter_state_output = TRUE;
     }
     else if ((!strcmp(argv[arg], "-p")) || (!strcmp(argv[arg], "--parseable-output")))
     {
-        dc->parseable_output = 1;
+        dc->parseable_output = TRUE;
     }
     else if ((!strcmp(argv[arg], "-c")) || (!strcmp(argv[arg], "--canonized-order-output")))
     {
-        dc->canonized_order_output = 1;
+        dc->canonized_order_output = TRUE;
     }
     else if ((!strcmp(argv[arg], "-t")) || (!strcmp(argv[arg], "--display-10-as-t")))
     {
-        dc->display_10_as_t = 1;
+        dc->display_10_as_t = TRUE;
     }
     else if ((!strcmp(argv[arg], "-m")) || (!strcmp(argv[arg], "--display-moves")))
     {
-        dc->display_moves = 1;
-        dc->display_states = 0;
+        dc->display_moves = TRUE;
+        dc->display_states = FALSE;
     }
     else if ((!strcmp(argv[arg], "-sn")) || (!strcmp(argv[arg], "--standard-notation")))
     {
-        dc->standard_notation = 1;
+        dc->standard_notation = TRUE;
     }
     else if ((!strcmp(argv[arg], "-sam")) || (!strcmp(argv[arg], "--display-states-and-moves")))
     {
-        dc->display_moves = 1;
-        dc->display_states = 1;
+        dc->display_moves = TRUE;
+        dc->display_states = TRUE;
     }
     else if ((!strcmp(argv[arg], "-pi")) || (!strcmp(argv[arg], "--display-parent-iter")))
     {
-        dc->display_parent_iter_num = 1;
+        dc->display_parent_iter_num = TRUE;
     }
     else
     {
@@ -184,7 +185,7 @@ static int cmd_line_callback(
 #define total_iterations_limit_per_board 100000
 #endif
 
-static char * known_parameters[] = {
+static const char * known_parameters[] = {
     "-i", "--iter-output",
     "-s", "--state-output",
     "-p", "--parseable-output",
@@ -415,6 +416,10 @@ int main(int argc, char * argv[])
     else
     {
         binary_output.file = NULL;
+        binary_output.buffer
+            = binary_output.ptr
+            = binary_output.buffer_end
+            = NULL;
     }
 
     user.instance = freecell_solver_user_alloc();
@@ -425,7 +430,7 @@ int main(int argc, char * argv[])
         freecell_solver_user_cmd_line_parse_args(
             user.instance,
             argc,
-            argv,
+            (freecell_solver_str_t *)argv,
             arg,
             known_parameters,
             cmd_line_callback,
@@ -497,6 +502,17 @@ int main(int argc, char * argv[])
             fflush(stdout);
             print_int_wrapper(-1);
         }
+        else if (ret == FCS_STATE_FLARES_PLAN_ERROR)
+        {
+            const char * error_string;
+
+            error_string =
+                freecell_solver_user_get_last_error_string(user.instance);
+
+            printf("Flares Plan: %s\n", error_string);
+
+            break;
+        }
         else if (ret == FCS_STATE_IS_NOT_SOLVEABLE)
         {
 #ifndef WIN32
